------------------------------------------------------------------------------
--
--  package Server_Communications (spec)
--
------------------------------------------------------------------------------
--  Standard packages:

------------------------------------------------------------------------------
--  Other packages:

with Base_Types;

------------------------------------------------------------------------------

package Server_Communications is

   ------------------------------------------------------------------------
   --  Initialisation:

   procedure Initialise (Width  :    out Base_Types.X_Coordinate;
                         Height :    out Base_Types.Y_Coordinate);

   procedure Get_Board (Width  : in     Base_Types.X_Coordinate;
                        Height : in     Base_Types.Y_Coordinate;
                        Board  :    out Base_Types.Board_Type);

   procedure Get_Robot_Configuration
     (Robot :    out Base_Types.Robot_configuration);

   ------------------------------------------------------------------------
   --  Server response:

   type Action_Results is (Moved_North, Moved_East, Moved_South, Moved_West,
                           Picked_Gift_Up, Dropped_Gift,
                           Set_XY);

   type Action_Result (Action : Action_Results := Moved_North) is
      record
         case Action is
            when Picked_Gift_Up | Dropped_Gift =>
               Gift : Base_Types.Gift_ID;
            when Set_XY =>
               X    : Base_Types.X_Coordinate;
               Y    : Base_Types.Y_Coordinate;
            when others =>
               null;
         end case;
      end record;

   procedure Receive_Action_Results;

   function More_Action_Results return Boolean;

   function Action_Robot_Change return Boolean;

   procedure Get_Action_Robot_Change (Robot :    out Base_Types.Robot_ID);

   procedure Get_Action_Result (Robot :    out Base_Types.Robot_ID;
                                Item  :    out Action_Result);

   ------------------------------------------------------------------------
   --  Available gifts:

   procedure Receive_Available_Gifts;

   function More_Available_Gifts return Boolean;

   procedure Get_Gift_Data (ID   :    out Base_Types.Gift_ID;
                            Data :    out Base_Types.Gift_Type);

   ------------------------------------------------------------------------
   --  Request action:

   procedure Request_Action (Bid  : in     Base_Types.Bid_Type;
                             Item : in     Base_Types.Command);

   ------------------------------------------------------------------------

end Server_Communications;
