------------------------------------------------------------------------------
--
--  procedure Socket_Cat (body)
--
------------------------------------------------------------------------------
--  Standard packages:

with Ada.Characters.Latin_1;
with Ada.Command_Line;
with Ada.Integer_Text_IO;
with Ada.Streams;
with Ada.Strings;
with Ada.Strings.Fixed;
with Ada.Strings.Unbounded;
with Ada.Text_IO;

------------------------------------------------------------------------------
--  Other packages:

with Sockets;

------------------------------------------------------------------------------

procedure Socket_Cat is

   ---------------------------------------------------------------------------
   --  Server connection:

   Server   : Sockets.Socket_FD;
   New_Line : constant Character := Character'Val (10);

   ---------------------------------------------------------------------------
   --  function Server_Name:

   function Server_Name return String is

      use Ada.Command_Line;
      use Ada.Text_IO;

      Default : constant String := "localhost";

   begin --  Server_Name
      if Argument_Count = 2 then
         return Argument (1);
      else
         Put_Line (File => Standard_Error,
                   Item => "Expected the server name as the first command " &
                           "line argument and the port number as the " &
                           "second command line argument. Using """ &
                           Default & """ as the server name.");
         return Default;
      end if;
   end Server_Name;

   ---------------------------------------------------------------------------
   --  function Server_Port_Number:

   function Server_Port_Number return Positive is

      use Ada.Command_Line;
      use Ada.Text_IO;

      Default : constant Positive := 2002;

   begin --  Server_Port_Number
      if Argument_Count = 2 then
         return Positive'Value (Argument (2));
      else
         Put_Line (File => Standard_Error,
                   Item => "Expected the server name as the first command " &
                           "line argument and the port number as the " &
                           "second command line argument. Using """ &
                           Positive'Image (Default) & """ as the server " &
                           "port number.");
         return Default;
      end if;
   exception
      when others =>
         Put_Line (File => Standard_Error,
                   Item => "The second command line argument was not a " &
                           "positive integer. Using """ &
                           Positive'Image (Default) & """ as the server " &
                           "port number.");
         return Default;
   end Server_Port_Number;

   ---------------------------------------------------------------------------
   --  function Get:

   function Get (Socket : Sockets.Socket_FD) return Character is
      C : Ada.Streams.Stream_Element_Array (0 .. 0);
   begin
      Sockets.Receive (Socket, C);
      return Character'Val (C (0));
   end Get;

   ---------------------------------------------------------------------------

   use Ada.Characters.Latin_1;
   use Ada.Integer_Text_IO;
   use Ada.Text_IO;
   use Sockets;

   Line   : Positive := 1;
   Char   : Character;

begin
   Socket (Sock => Server);
   Connect (Socket => Server,
            Host   => Server_Name,
            Port   => Server_Port_Number);
   Unset_Buffer (Socket => Server);

   Put (Line);
   Put (": """);

   loop
      Char := Get (Socket => Server);

      if Char = LF then
         Put_Line ("""");
         Line := Line + 1;
         Put (Line);
         Put (": """);
      elsif Char /= CR then
         Put (Char);
      end if;
   end loop;
end Socket_Cat;
